@@ -1,21 +1,22 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
- * Copyright (c) 2009-2010 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * This program is free software; you can redistribute it and/or 
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of 
+ * the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public 
+ * License along with this program; if not, write to the Free 
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -69,7 +70,7 @@ thunar_dnd_ask (GtkWidget    *widget,
 {
   static const GdkDragAction dnd_action_items[] = { GDK_ACTION_COPY, GDK_ACTION_MOVE, GDK_ACTION_LINK };
   static const gchar        *dnd_action_names[] = { N_ ("_Copy here"), N_ ("_Move here"), N_ ("_Link here") };
-  static const gchar        *dnd_action_icons[] = { GTK_STOCK_COPY, "folder-move", NULL };
+  static const gchar        *dnd_action_icons[] = { "stock_folder-copy", "stock_folder-move", NULL };
 
   ThunarxProviderFactory *factory;
   GdkDragAction           dnd_action = 0;
@@ -108,7 +109,6 @@ thunar_dnd_ask (GtkWidget    *widget,
           {
             image = gtk_image_new_from_icon_name (dnd_action_icons[n], GTK_ICON_SIZE_MENU);
             gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-            gtk_widget_show (image);
           }
       }
 
@@ -119,7 +119,7 @@ thunar_dnd_ask (GtkWidget    *widget,
 
   /* determine the toplevel window the widget belongs to */
   window = gtk_widget_get_toplevel (widget);
-  if (G_LIKELY (window != NULL && GTK_WIDGET_TOPLEVEL (window)))
+  if (G_LIKELY (window != NULL && gtk_widget_get_toplevel (window)))
     {
       /* check if we can resolve all paths */
       for (lp = path_list; lp != NULL; lp = lp->next)
@@ -179,7 +179,7 @@ thunar_dnd_ask (GtkWidget    *widget,
 
   /* cleanup */
   g_object_unref (G_OBJECT (factory));
-  g_list_free (file_list);
+  g_list_free_full (file_list, g_object_unref);
 
   return dnd_action;
 }
@@ -217,7 +217,7 @@ thunar_dnd_perform (GtkWidget    *widget,
 
   _thunar_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
-  _thunar_return_val_if_fail (GTK_WIDGET_REALIZED (widget), FALSE);
+  _thunar_return_val_if_fail (gtk_widget_get_realized (widget), FALSE);
 
   /* query a reference on the application object */
   application = thunar_application_get ();
@@ -247,7 +247,7 @@ thunar_dnd_perform (GtkWidget    *widget,
   else if (thunar_file_is_executable (file))
     {
       /* TODO any chance to determine the working dir here? */
-      succeed = thunar_file_execute (file, NULL, gtk_widget_get_screen (widget), file_list, &error);
+      succeed = thunar_file_execute (file, NULL, widget, file_list, &error);
       if (G_UNLIKELY (!succeed))
         {
           /* display an error to the user */
