@@ -34,6 +34,7 @@
 
 #include <thunar/thunar-file.h>
 #include <thunar/thunar-gio-extensions.h>
+#include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-util.h>
 
@@ -77,7 +78,7 @@ thunar_g_file_new_for_bookmarks (void)
   gchar *filename;
   GFile *bookmarks;
 
-  filename = g_build_filename (xfce_get_homedir (), ".gtk-bookmarks", NULL);
+  filename = g_build_filename (g_get_user_config_dir (), "gtk-3.0", "bookmarks", NULL);
   bookmarks = g_file_new_for_path (filename);
   g_free (filename);
 
@@ -410,21 +411,27 @@ thunar_g_file_get_free_space (GFile   *file,
 gchar *
 thunar_g_file_get_free_space_string (GFile *file)
 {
-  gchar   *fs_free_str;
-  gchar   *fs_size_str;
-  guint64  fs_free;
-  guint64  fs_size;
-  gchar   *fs_string = NULL;
+  gchar             *fs_free_str;
+  gchar             *fs_size_str;
+  guint64            fs_free;
+  guint64            fs_size;
+  gchar             *fs_string = NULL;
+  ThunarPreferences *preferences;
+  gboolean           file_size_binary;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
 
+  preferences = thunar_preferences_get ();
+  g_object_get (preferences, "misc-file-size-binary", &file_size_binary, NULL);
+  g_object_unref (preferences);
+
   if (thunar_g_file_get_free_space (file, &fs_free, &fs_size)
       && fs_size > 0)
     {
-      fs_free_str = g_format_size (fs_free);
-      fs_size_str = g_format_size (fs_size);
+      fs_free_str = g_format_size_full (fs_free, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
+      fs_size_str = g_format_size_full (fs_size, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
       /* free disk space string */
-      fs_string = g_strdup_printf (_("%s of %s (%d%% used)"),
+      fs_string = g_strdup_printf (_("%s of %s free (%d%% used)"),
                                    fs_free_str, fs_size_str,
                                    (gint) ((fs_size - fs_free) * 100 / fs_size));
       g_free (fs_free_str);
