@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -661,7 +661,7 @@ thunar_history_show_menu (GtkAction     *action,
           else if (thunar_g_file_is_root (lp->data))
             icon_name = "drive-harddisk";
           else
-            icon_name = GTK_STOCK_DIRECTORY;
+            icon_name = "folder";
 
           image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
         }
