@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
@@ -815,8 +815,9 @@ thunar_launcher_update_idle (gpointer data)
        ** - "Open", "Open in n New Windows" and "Open in n New Tabs" actions
        **/
 
-      /* Prepare "Open" label */
+      /* Prepare "Open" label and icon */
       gtk_action_set_label (launcher->action_open, _("_Open"));
+      gtk_action_set_stock_id (launcher->action_open, GTK_STOCK_OPEN);
 
       if (n_selected_files == n_directories && n_directories >= 1)
         {
@@ -928,6 +929,7 @@ thunar_launcher_update_idle (gpointer data)
           /* turn the "Open" action into "Execute" */
           g_object_set (G_OBJECT (launcher->action_open),
                         "label", _("_Execute"),
+                        "stock-id", GTK_STOCK_EXECUTE,
                         "tooltip", ngettext ("Execute the selected file", "Execute the selected files", n_selected_files),
                         NULL);
         }
@@ -945,6 +947,10 @@ thunar_launcher_update_idle (gpointer data)
           g_free (tooltip);
           g_free (label);
 
+          /* load default application icon */
+          gtk_action_set_stock_id (launcher->action_open, NULL);
+          gtk_action_set_gicon (launcher->action_open, g_app_info_get_icon (applications->data));
+
           /* remember the default application for the "Open" action */
           g_object_set_qdata_full (G_OBJECT (launcher->action_open), thunar_launcher_handler_quark, applications->data, g_object_unref);
 
@@ -1051,13 +1057,13 @@ thunar_launcher_update_idle (gpointer data)
 
       /* FIXME Add desktop actions here. Unfortunately they are not supported by
        * GIO, so we'll have to roll our own thing here */
+    }
 
-      /* schedule an update of the "Send To" menu */
-      if (G_LIKELY (launcher->sendto_idle_id == 0))
-        {
-          launcher->sendto_idle_id = g_idle_add_full (G_PRIORITY_LOW, thunar_launcher_sendto_idle,
-                                                      launcher, thunar_launcher_sendto_idle_destroy);
-        }
+  /* schedule an update of the "Send To" menu */
+  if (G_LIKELY (launcher->sendto_idle_id == 0))
+    {
+      launcher->sendto_idle_id = g_idle_add_full (G_PRIORITY_LOW, thunar_launcher_sendto_idle,
+                                                  launcher, thunar_launcher_sendto_idle_destroy);
     }
 
   GDK_THREADS_LEAVE ();
