@@ -138,6 +138,8 @@ struct _ThunarPropertiesDialog
   GtkWidget              *deleted_label;
   GtkWidget              *modified_label;
   GtkWidget              *accessed_label;
+  GtkWidget              *freespace_vbox;
+  GtkWidget              *freespace_bar;
   GtkWidget              *freespace_label;
   GtkWidget              *volume_image;
   GtkWidget              *volume_label;
@@ -231,7 +233,7 @@ thunar_properties_dialog_init (ThunarPropertiesDialog *dialog)
                           GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                           GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                           NULL);
-  gtk_window_set_default_size (GTK_WINDOW (dialog), 400, 430);
+  gtk_window_set_default_size (GTK_WINDOW (dialog), 500, 550);
 
   dialog->notebook = gtk_notebook_new ();
   gtk_container_set_border_width (GTK_CONTAINER (dialog->notebook), 6);
@@ -491,20 +493,28 @@ thunar_properties_dialog_init (ThunarPropertiesDialog *dialog)
 
   ++row;
 
-  label = gtk_label_new (_("Free Space:"));
+  label = gtk_label_new (_("Usage:"));
   gtk_label_set_attributes (GTK_LABEL (label), thunar_pango_attr_list_bold ());
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.0f);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 3);
   gtk_widget_show (label);
 
+  dialog->freespace_vbox = gtk_vbox_new (FALSE, 4);
+  gtk_table_attach (GTK_TABLE (table), dialog->freespace_vbox, 1, 2, row, row + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 3);
+  exo_binding_new (G_OBJECT (dialog->freespace_vbox), "visible", G_OBJECT (label), "visible");
+  gtk_widget_show (dialog->freespace_vbox);
+
   dialog->freespace_label = g_object_new (GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
   gtk_label_set_selectable (GTK_LABEL (dialog->freespace_label), TRUE);
-  exo_binding_new (G_OBJECT (dialog->freespace_label), "visible", G_OBJECT (label), "visible");
-  gtk_table_attach (GTK_TABLE (table), dialog->freespace_label, 1, 2, row, row + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 3);
+  gtk_box_pack_start (GTK_BOX (dialog->freespace_vbox), dialog->freespace_label, TRUE, TRUE, 0);
   gtk_widget_show (dialog->freespace_label);
 
-  ++row;
+  dialog->freespace_bar = g_object_new (GTK_TYPE_PROGRESS_BAR, NULL);
+  gtk_box_pack_start (GTK_BOX (dialog->freespace_vbox), dialog->freespace_bar, TRUE, TRUE, 0);
+  gtk_widget_set_size_request (dialog->freespace_bar, -1, 10);
+  gtk_widget_show (dialog->freespace_bar);
 
+  ++row;
 
   spacer = g_object_new (GTK_TYPE_ALIGNMENT, "height-request", 12, NULL);
   gtk_table_attach (GTK_TABLE (table), spacer, 0, 2, row, row + 1, GTK_FILL, GTK_FILL, 0, 3);
@@ -637,7 +647,7 @@ thunar_properties_dialog_response (GtkDialog *dialog,
     {
       xfce_dialog_show_help (GTK_WINDOW (dialog), "thunar",
                              "working-with-files-and-folders",
-                             "file-properties");
+                             "file_properties");
     }
   else if (GTK_DIALOG_CLASS (thunar_properties_dialog_parent_class)->response != NULL)
     {
@@ -668,6 +678,12 @@ thunar_properties_dialog_rename_error (ExoJob                 *job,
   _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
   _thunar_return_if_fail (g_list_length (dialog->files) == 1);
 
+  /* reset the entry display name to the original name, so the focus
+     out event does not trigger the rename again by calling
+     thunar_properties_dialog_name_activate */
+  gtk_entry_set_text (GTK_ENTRY (dialog->name_entry),
+                      thunar_file_get_display_name (THUNAR_FILE (dialog->files->data)));
+
   /* display an error message */
   thunar_dialogs_show_error (GTK_WIDGET (dialog), error, _("Failed to rename \"%s\""),
                              thunar_file_get_display_name (THUNAR_FILE (dialog->files->data)));
@@ -858,6 +874,9 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
   ThunarFile        *file;
   ThunarFile        *parent_file;
   gboolean           show_chooser;
+  guint64            fs_free;
+  guint64            fs_size;
+  gdouble            fs_fraction = 0.0;
 
   _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
   _thunar_return_if_fail (g_list_length (dialog->files) == 1);
@@ -1037,20 +1056,27 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
   if (thunar_file_is_directory (file))
     {
       fs_string = thunar_g_file_get_free_space_string (thunar_file_get_file (file));
+      if (thunar_g_file_get_free_space (thunar_file_get_file (file), &fs_free, &fs_size)
+          && fs_size > 0)
+        {
+          /* free disk space fraction */
+          fs_fraction = ((fs_size - fs_free) * 100 / fs_size);
+        }
       if (fs_string != NULL)
         {
           gtk_label_set_text (GTK_LABEL (dialog->freespace_label), fs_string);
-          gtk_widget_show (dialog->freespace_label);
+          gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (dialog->freespace_bar), fs_fraction / 100);
+          gtk_widget_show (dialog->freespace_vbox);
           g_free (fs_string);
         }
       else
         {
-          gtk_widget_hide (dialog->freespace_label);
+          gtk_widget_hide (dialog->freespace_vbox);
         }
     }
   else
     {
-      gtk_widget_hide (dialog->freespace_label);
+      gtk_widget_hide (dialog->freespace_vbox);
     }
 
   /* update the volume */
@@ -1107,7 +1133,7 @@ thunar_properties_dialog_update_multiple (ThunarPropertiesDialog *dialog)
   gtk_widget_hide (dialog->deleted_label);
   gtk_widget_hide (dialog->modified_label);
   gtk_widget_hide (dialog->accessed_label);
-  gtk_widget_hide (dialog->freespace_label);
+  gtk_widget_hide (dialog->freespace_vbox);
   gtk_widget_hide (dialog->origin_label);
   gtk_widget_hide (dialog->openwith_chooser);
   gtk_widget_hide (dialog->link_label);
