@@ -113,6 +113,9 @@ static gboolean       thunar_shortcuts_view_selection_func               (GtkTre
                                                                           GtkTreePath              *path,
                                                                           gboolean                  path_currently_selected,
                                                                           gpointer                  user_data);
+static void           thunar_shortcuts_view_context_menu_visibility      (ThunarShortcutsView      *view,
+                                                                          GdkEventButton           *event,
+                                                                          GtkTreeModel             *model);
 static void           thunar_shortcuts_view_context_menu                 (ThunarShortcutsView      *view,
                                                                           GdkEventButton           *event,
                                                                           GtkTreeModel             *model,
@@ -159,8 +162,10 @@ struct _ThunarShortcutsViewClass
 struct _ThunarShortcutsView
 {
   GtkTreeView        __parent__;
-  ThunarPreferences *preferences;
-  GtkCellRenderer   *icon_renderer;
+
+
+  ThunarPreferences      *preferences;
+  GtkCellRenderer        *icon_renderer;
 
   ThunarxProviderFactory *provider_factory;
 
@@ -168,7 +173,7 @@ struct _ThunarShortcutsView
    * button-press-event handler if the associated
    * button-release-event should activate.
    */
-  gint pressed_button;
+  gint  pressed_button;
   guint pressed_eject_button : 1;
 
   /* drop site support */
@@ -179,7 +184,7 @@ struct _ThunarShortcutsView
   /* id of the signal used to queue a resize on the
    * column whenever the shortcuts icon size is changed.
    */
-  gint queue_resize_signal_id;
+  gulong queue_resize_signal_id;
 };
 
 
@@ -296,14 +301,13 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
   /* header */
   renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
                            "weight", PANGO_WEIGHT_BOLD,
-                           "weight-set", TRUE,
                            "xpad", 6,
                            "ypad", 6,
                            NULL);
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
   gtk_tree_view_column_set_attributes (column, renderer,
                                        "text", THUNAR_SHORTCUTS_MODEL_COLUMN_NAME,
-                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER,
+                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER,
                                        NULL);
 
   /* separator for indent */
@@ -311,7 +315,7 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
   g_object_set (G_OBJECT (renderer), "xpad", 6, NULL);
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
   gtk_tree_view_column_set_attributes (column, renderer,
-                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM,
+                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM,
                                        NULL);
 
   /* allocate the special icon renderer */
@@ -321,7 +325,7 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
                                        "gicon", THUNAR_SHORTCUTS_MODEL_COLUMN_GICON,
                                        "file", THUNAR_SHORTCUTS_MODEL_COLUMN_FILE,
                                        "device", THUNAR_SHORTCUTS_MODEL_COLUMN_DEVICE,
-                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM,
+                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM,
                                        NULL);
 
   /* sync the "emblems" property of the icon renderer with the "shortcuts-icon-emblems" preference
@@ -332,14 +336,13 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
 
   /* allocate the text renderer (ellipsizing as required, but "File System" must fit) */
   renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
-                           "ellipsize-set", TRUE,
                            "ellipsize", PANGO_ELLIPSIZE_END,
                            NULL);
   g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (thunar_shortcuts_view_renamed), view);
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
   gtk_tree_view_column_set_attributes (column, renderer,
                                        "text", THUNAR_SHORTCUTS_MODEL_COLUMN_NAME,
-                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM,
+                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM,
                                        NULL);
 
   /* spinner to indicate (un)mount/eject delay */
@@ -421,14 +424,15 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
   /* let the widget process the event first (handles focussing and scrolling) */
   result = (*GTK_WIDGET_CLASS (thunar_shortcuts_view_parent_class)->button_press_event) (widget, event);
 
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+
   /* resolve the path at the cursor position */
   if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL))
     {
       /* check if we should popup the context menu */
-      if (G_UNLIKELY (event->button == 3 && event->type == GDK_BUTTON_PRESS))
+      if (G_UNLIKELY (event->button == 3))
         {
           /* determine the iterator for the path */
-          model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
           if (gtk_tree_model_get_iter (model, &iter, path))
             {
               /* popup the context menu */
@@ -438,14 +442,12 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
               result = TRUE;
             }
         }
-      else if ((event->button == 1 || event->button == 2)
-               && event->type == GDK_BUTTON_PRESS
-               && (event->state & gtk_accelerator_get_default_mod_mask ()) == 0)
+      else if (event->button == 1 || event->button == 2)
         {
           /* check if we clicked the eject button area */
           column_width = gtk_tree_view_column_get_width (gtk_tree_view_get_column (GTK_TREE_VIEW (view), 0));
           gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &icon_width, &icon_height);
-          if (event->x >= column_width - icon_width - 3)
+          if (event->button == 1 && event->x >= column_width - icon_width - 3)
             {
               /* check if that shortcut actually has an eject button */
               model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
@@ -456,6 +458,8 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
                     view->pressed_eject_button = 1;
                 }
             }
+          else
+              gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
 
           /* remember the button as pressed and handle it in the release handler */
           view->pressed_button = event->button;
@@ -464,6 +468,11 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
       /* release the path */
       gtk_tree_path_free (path);
     }
+  else if (event->button == 3)
+    {
+      thunar_shortcuts_view_context_menu_visibility (view, event, model);
+      result = TRUE;
+    }
 
   return result;
 }
@@ -494,6 +503,11 @@ thunar_shortcuts_view_button_release_event (GtkWidget      *widget,
         {
           /* button 2 opens in a new window or tab */
           g_object_get (view->preferences, "misc-middle-click-in-tab", &in_tab, NULL);
+
+          /* holding ctrl inverts the action */
+          if ((event->state & GDK_CONTROL_MASK) != 0)
+            in_tab = !in_tab;
+
           thunar_shortcuts_view_open (view, in_tab ? OPEN_IN_TAB : OPEN_IN_WINDOW);
         }
     }
@@ -912,15 +926,15 @@ thunar_shortcuts_view_selection_func (GtkTreeSelection *selection,
 
   /* don't allow selecting headers */
   gtk_tree_model_get_iter (model, &iter, path);
-  gtk_tree_model_get (model, &iter, THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER, &is_header, -1);
+  gtk_tree_model_get (model, &iter, THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER, &is_header, -1);
   return !is_header;
 }
 
 
 
 static void
-thunar_shortcuts_view_context_menu_header_toggled (GtkCheckMenuItem *item,
-                                                   GtkTreeModel     *model)
+thunar_shortcuts_view_context_menu_visibility_toggled (GtkCheckMenuItem *item,
+                                                       GtkTreeModel     *model)
 {
   gboolean             hidden;
   GtkTreePath         *path;
@@ -942,10 +956,9 @@ thunar_shortcuts_view_context_menu_header_toggled (GtkCheckMenuItem *item,
 
 
 static void
-thunar_shortcuts_view_context_menu_header (ThunarShortcutsView *view,
-                                           GdkEventButton      *event,
-                                           GtkTreeModel        *model,
-                                           GtkTreeIter         *header_iter)
+thunar_shortcuts_view_context_menu_visibility (ThunarShortcutsView *view,
+                                               GdkEventButton      *event,
+                                               GtkTreeModel        *model)
 {
   GtkTreeIter          iter;
   guint                mask = 0;
@@ -957,17 +970,18 @@ thunar_shortcuts_view_context_menu_header (ThunarShortcutsView *view,
   gboolean             visible;
   GtkTreePath         *path;
   GtkTreeModel        *child_model;
+  GtkWidget           *submenu;
 
   /* unselect the items */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
   gtk_tree_selection_unselect_all (GTK_TREE_SELECTION (selection));
 
   /* prepare the popup menu */
-  menu = gtk_menu_new ();
+  menu = submenu = gtk_menu_new ();
 
   /* process all items below the header */
   child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
-  gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &iter, header_iter);
+  gtk_tree_model_get_iter_first (GTK_TREE_MODEL (child_model), &iter);
   path = gtk_tree_model_get_path (child_model, &iter);
   do
     {
@@ -978,7 +992,7 @@ thunar_shortcuts_view_context_menu_header (ThunarShortcutsView *view,
                           THUNAR_SHORTCUTS_MODEL_COLUMN_VISIBLE, &visible,
                           -1);
 
-      if (mask == 0 && (group & THUNAR_SHORTCUT_GROUP_HEADER) != 0)
+      if ((group & THUNAR_SHORTCUT_GROUP_HEADER) != 0)
         {
           /* get the mask of the group */
           if ((group & THUNAR_SHORTCUT_GROUP_DEVICES) != 0)
@@ -993,23 +1007,21 @@ thunar_shortcuts_view_context_menu_header (ThunarShortcutsView *view,
           /* create menu items */
           mi = gtk_menu_item_new_with_label (label);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-          gtk_widget_set_sensitive (mi, FALSE);
           gtk_widget_show (mi);
 
-          mi = gtk_separator_menu_item_new ();
-          gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-          gtk_widget_show (mi);
+          submenu = gtk_menu_new ();
+          gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), submenu);
         }
       else if ((group & mask) != 0)
         {
           mi = gtk_check_menu_item_new_with_label (label);
-          gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+          gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mi);
           gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mi), visible);
           g_object_set_data_full (G_OBJECT (mi), I_("thunar-shortcuts-row"),
                                   gtk_tree_row_reference_new (child_model, path),
                                   (GDestroyNotify) gtk_tree_row_reference_free);
           g_signal_connect (G_OBJECT (mi), "toggled",
-            G_CALLBACK (thunar_shortcuts_view_context_menu_header_toggled), child_model);
+            G_CALLBACK (thunar_shortcuts_view_context_menu_visibility_toggled), child_model);
           gtk_widget_show (mi);
         }
 
@@ -1048,12 +1060,15 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
   gboolean             is_header;
   GFile               *mount_point;
   GtkTreeModel        *shortcuts_model;
+  gboolean             can_mount;
+  gboolean             can_unmount;
+  gboolean             can_eject;
 
   /* check if this is an item menu or a header menu */
-  gtk_tree_model_get (model, iter, THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER, &is_header, -1);
+  gtk_tree_model_get (model, iter, THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER, &is_header, -1);
   if (is_header)
     {
-      thunar_shortcuts_view_context_menu_header (view, event, model, iter);
+      thunar_shortcuts_view_context_menu_visibility (view, event, model);
       return;
     }
 
@@ -1098,6 +1113,13 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
   switch (group)
     {
       case THUNAR_SHORTCUT_GROUP_DEVICES_VOLUMES:
+        can_mount = thunar_device_can_mount (device);
+        can_unmount = thunar_device_can_unmount (device);
+        can_eject = thunar_device_can_eject (device);
+
+        if (!can_mount && !can_unmount && !can_eject)
+          break;
+
         /* append a menu separator */
         item = gtk_separator_menu_item_new ();
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -1105,19 +1127,19 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
 
         /* append the "Mount" item */
         item = gtk_image_menu_item_new_with_mnemonic (_("_Mount"));
-        gtk_widget_set_visible (item, thunar_device_can_mount (device));
+        gtk_widget_set_visible (item, can_mount);
         g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_shortcuts_view_mount), view);
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
         /* append the "Unmount" item */
         item = gtk_image_menu_item_new_with_mnemonic (_("_Unmount"));
-        gtk_widget_set_visible (item, thunar_device_can_unmount (device));
+        gtk_widget_set_visible (item, can_unmount);
         g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_shortcuts_view_unmount), view);
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
         /* append the "Disconnect" (eject + safely remove drive) item */
         item = gtk_image_menu_item_new_with_mnemonic (_("_Eject"));
-        gtk_widget_set_visible (item, thunar_device_can_eject (device));
+        gtk_widget_set_visible (item, can_eject);
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_shortcuts_view_eject), view);
         break;
@@ -1226,8 +1248,7 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
     }
 
   /* append the remove menu item */
-  if (group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-      || group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS)
+  if (group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
     {
       /* append a menu separator */
       item = gtk_separator_menu_item_new ();
@@ -1856,7 +1877,6 @@ thunar_shortcuts_view_eject (ThunarShortcutsView *view)
   GtkTreeIter       iter;
   ThunarDevice     *device;
   GMountOperation  *mount_operation;
-  GtkWidget        *window;
   GtkTreeModel     *child_model;
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (view));
@@ -1870,8 +1890,7 @@ thunar_shortcuts_view_eject (ThunarShortcutsView *view)
       _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
 
       /* prepare a mount operation */
-      window = gtk_widget_get_toplevel (GTK_WIDGET (view));
-      mount_operation = gtk_mount_operation_new (GTK_WINDOW (window));
+      mount_operation = thunar_gtk_mount_operation_new (GTK_WIDGET (view));
 
       /* start the spinner */
       child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
@@ -2000,7 +2019,6 @@ thunar_shortcuts_view_unmount (ThunarShortcutsView *view)
   GtkTreeIter       iter;
   ThunarDevice     *device;
   GMountOperation  *mount_operation;
-  GtkWidget        *window;
   GtkTreeModel     *child_model;
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (view));
@@ -2014,8 +2032,7 @@ thunar_shortcuts_view_unmount (ThunarShortcutsView *view)
       _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
 
       /* prepare a mount operation */
-      window = gtk_widget_get_toplevel (GTK_WIDGET (view));
-      mount_operation = gtk_mount_operation_new (GTK_WINDOW (window));
+      mount_operation = thunar_gtk_mount_operation_new (GTK_WIDGET (view));
 
       /* start the spinner */
       child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
