@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -126,7 +126,6 @@ struct _ThunarStandardView
 
   ThunarClipboardManager    *clipboard;
   ThunarListModel           *model;
-  gchar                     *statusbar_text;
 
   GtkActionGroup            *action_group;
   GtkUIManager              *ui_manager;
